@@ -1,11 +1,13 @@
 /*
  * This is free software, licensed under the Gnu Public License (GPL)
  * get a copy from <http://www.gnu.org/licenses/gpl.html>
- * $Id: EchoCommandProperty.java,v 1.1 2004/02/01 16:39:09 hzeller Exp $
+ * $Id: EchoCommandProperty.java,v 1.4 2004-03-07 14:22:02 hzeller Exp $
  * author: Henner Zeller <H.Zeller@acm.org>
  */
 package henplus;
 
+import henplus.event.ExecutionListener;
+
 import henplus.property.BooleanPropertyHolder;
 
 /**
@@ -43,8 +45,9 @@ public final class EchoCommandProperty
     //-- Execution listener
 
     public void beforeExecution(SQLSession session, String command) {
-        HenPlus.msg().println(command);
+        HenPlus.msg().println(command.trim());
     }
     public void afterExecution(SQLSession session, String command, int result){
+        /* don't care */
     }
 }
