@@ -1,29 +1,59 @@
 /*
  * This is free software, licensed under the Gnu Public License (GPL)
  * get a copy from <http://www.gnu.org/licenses/gpl.html>
- * $Id: HenPlus.java,v 1.64 2004/02/02 13:36:37 hzeller Exp $
+ * $Id: HenPlus.java,v 1.77 2008-10-19 09:14:49 hzeller Exp $
  * author: Henner Zeller <H.Zeller@acm.org>
  */
 package henplus;
 
-import henplus.commands.*;
-import henplus.commands.properties.*;
-import henplus.view.util.Terminal;
+import henplus.commands.AboutCommand;
+import henplus.commands.AliasCommand;
+import henplus.commands.ConnectCommand;
+import henplus.commands.DescribeCommand;
+import henplus.commands.DriverCommand;
+import henplus.commands.DumpCommand;
+import henplus.commands.EchoCommand;
+import henplus.commands.ExitCommand;
+import henplus.commands.HelpCommand;
+import henplus.commands.ImportCommand;
+import henplus.commands.KeyBindCommand;
+import henplus.commands.ListUserObjectsCommand;
+import henplus.commands.LoadCommand;
+import henplus.commands.PluginCommand;
+import henplus.commands.SQLCommand;
+import henplus.commands.SetCommand;
+import henplus.commands.ShellCommand;
+import henplus.commands.SpoolCommand;
+import henplus.commands.StatusCommand;
+import henplus.commands.SystemInfoCommand;
+import henplus.commands.TreeCommand;
+import henplus.commands.properties.PropertyCommand;
+import henplus.commands.properties.SessionPropertyCommand;
+import henplus.io.ConfigurationContainer;
 
 import java.io.BufferedReader;
 import java.io.EOFException;
 import java.io.File;
+import java.util.Iterator;
 import java.io.IOException;
+import java.io.InputStream;
+import org.apache.commons.cli.CommandLine;
+import org.apache.commons.cli.CommandLineParser;
+import org.apache.commons.cli.HelpFormatter;
+import org.apache.commons.cli.Option;
+import org.apache.commons.cli.Options;
+import org.apache.commons.cli.ParseException;
+import org.apache.commons.cli.PosixParser;
 import java.io.InputStreamReader;
+import java.io.OutputStream;
 import java.util.Map;
-import java.io.PrintStream;
 
 import org.gnu.readline.Readline;
 import org.gnu.readline.ReadlineLibrary;
 
 public class HenPlus implements Interruptable {
+    private static final String HISTORY_NAME = "history";
     public static final boolean verbose = false; // debug.
-    private static final String EXIT_MSG   = "good bye.";
     private static final String HENPLUSDIR = ".henplus";
     private static final String PROMPT     = "Hen*Plus> ";
 
@@ -38,7 +68,8 @@ public class HenPlus implements Interruptable {
     private final StringBuffer          _historyLine;
 
     private final boolean               _quiet;
-
+    private final ConfigurationContainer _historyConfig;
+    
     private SetCommand             _settingStore;
     private SessionManager         _sessionManager;
     private CommandDispatcher      _dispatcher;
@@ -90,17 +121,16 @@ public class HenPlus implements Interruptable {
                       new PrintStreamOutputDevice(System.err));
         }
 
-	// output of special characters.
-	Terminal.setTerminalAvailable(_fromTerminal);
-
 	if (!_quiet) {
 	    System.err.println("using GNU readline (Brian Fox, Chet Ramey), Java wrapper by Bernhard Bablok");
 	}
+        _historyConfig = createConfigurationContainer(HISTORY_NAME);
 	Readline.initReadline("HenPlus");
-	try {
-	    HistoryWriter.readReadlineHistory(getHistoryLocation());
-	}
-	catch (Exception ignore) {}
+	_historyConfig.read(new ConfigurationContainer.ReadAction() {
+            public void readConfiguration(InputStream in) throws Exception {
+                HistoryWriter.readReadlineHistory(in);
+            }
+        });
 	
 	Readline.setWordBreakCharacters(" ,/()<>=\t\n"); // TODO..
 	setDefaultPrompt();
@@ -112,12 +142,12 @@ public class HenPlus implements Interruptable {
             .registerProperty("comments-remove",
                               _commandSeparator.getRemoveCommentsProperty());
 
-        _sessionManager = SessionManager.getInstance(this); 
-        
+        _sessionManager = SessionManager.getInstance(); 
+
         // FIXME: to many cross dependencies of commands now. clean up.
 	_settingStore = new SetCommand(this);
+        _dispatcher = new CommandDispatcher(_settingStore);
 	_objectLister = new ListUserObjectsCommand(this);
-	_dispatcher = new CommandDispatcher(_settingStore);
         _henplusProperties
             .registerProperty("echo-commands", 
                               new EchoCommandProperty(_dispatcher));
@@ -136,26 +166,28 @@ public class HenPlus implements Interruptable {
 	_dispatcher.register(new DriverCommand(this));
 	AliasCommand aliasCommand = new AliasCommand(this);
 	_dispatcher.register(aliasCommand);
+        if (_fromTerminal) {
+            _dispatcher.register(new KeyBindCommand(this));
+        }
+
         LoadCommand loadCommand = new LoadCommand();
 	_dispatcher.register(loadCommand);
 
-	_dispatcher.register(new ConnectCommand( argv, this, _sessionManager ));
+	_dispatcher.register(new ConnectCommand( this, _sessionManager ));
 	_dispatcher.register(new StatusCommand());
 
 	_dispatcher.register(_objectLister);
 	_dispatcher.register(new DescribeCommand(_objectLister));
         
-        /*** experimental ***/
 	_dispatcher.register(new TreeCommand(_objectLister));
 
 	_dispatcher.register(new SQLCommand(_objectLister, _henplusProperties));
 
-	//_dispatcher.register(new ImportCommand());
+	_dispatcher.register(new ImportCommand(_objectLister));
 	//_dispatcher.register(new ExportCommand());
 	_dispatcher.register(new DumpCommand(_objectLister, loadCommand));
 
 	_dispatcher.register(new ShellCommand());
-
 	_dispatcher.register(new SpoolCommand(this));
 	_dispatcher.register(_settingStore);
 
@@ -163,13 +195,50 @@ public class HenPlus implements Interruptable {
         propertyCommand = new PropertyCommand(this, _henplusProperties);
         _dispatcher.register(propertyCommand);
         _dispatcher.register(new SessionPropertyCommand(this));
-        
+    
+        _dispatcher.register( new SystemInfoCommand() );
+   
+    
 	pluginCommand.load();
 	aliasCommand.load();
         propertyCommand.load();
+        
+        Options availableOptions = new Options();
+        availableOptions.addOption(new Option("h", "help", false, "print this message"));
+        for (Iterator it = _dispatcher.getRegisteredCommands(); it.hasNext();) {
+            Command element = (Command) it.next();
+            try {
+                element.registerOptions(availableOptions);
+            }
+            catch (Throwable e) {
+                System.err.println("while registering "+element);
+                e.printStackTrace();
+            }
+        }
+        CommandLineParser parser = new PosixParser();
+        CommandLine line = null;
+        try {
+            line = parser.parse(availableOptions, argv);
+            for (Iterator it = _dispatcher.getRegisteredCommands(); it.hasNext();) {
+                Command element = (Command) it.next();
+		element.setOptions(availableOptions);
+		element.handleCommandline(line);
+            }
+        } 
+        catch (Exception e) {
+            System.err.println(e.getMessage());
+            line = null;
+        }
 
+        if (line == null || line.hasOption("h")) {
+            HelpFormatter formatter = new HelpFormatter();
+            formatter.printHelp( "henplus", availableOptions );
+            System.exit(0);
+        }
+        
 	Readline.setCompleter( _dispatcher );
 
+        /* FIXME: do this platform independently */
 	// in case someone presses Ctrl-C
 	try {
 	    Runtime.getRuntime()
@@ -190,7 +259,12 @@ public class HenPlus implements Interruptable {
 	 * if your compiler/system/whatever does not support the sun.misc.*
 	 * classes, then just disable this call and the SigIntHandler class.
 	 */
-        SigIntHandler.install();
+        try {
+            SigIntHandler.install();
+        }
+        catch (Throwable t) {
+            // ignore.
+        }
 
         /* TESTING  for ^Z support in the shell.
         sun.misc.SignalHandler stoptest = new sun.misc.SignalHandler () {
@@ -406,10 +480,11 @@ public class HenPlus implements Interruptable {
 	    if (_dispatcher != null) {
 		_dispatcher.shutdown();
 	    }
-	    try {
-		HistoryWriter.writeReadlineHistory(getHistoryLocation());
-	    }
-	    catch (Exception ignore) {}
+	    _historyConfig.write(new ConfigurationContainer.WriteAction() {
+	        public void writeConfiguration(OutputStream out) throws Exception {
+	            HistoryWriter.writeReadlineHistory(out);
+	        }
+	    });
 	    Readline.cleanup();
 	}
 	finally {
@@ -433,7 +508,7 @@ public class HenPlus implements Interruptable {
     /**
      * Provides access to the session manager. He maintains the list of opened
      * sessions with their names.
-     * @return
+     * @return the session manager.
      */
     public SessionManager getSessionManager() {
         return _sessionManager;
@@ -492,8 +567,7 @@ public class HenPlus implements Interruptable {
         StringBuffer result = new StringBuffer();
         String      varname;
         boolean     hasBrace= false;
-        boolean     knownVar= false;
-        
+                
         if (in == null) {
             return null;
         }
@@ -608,7 +682,19 @@ public class HenPlus implements Interruptable {
         System.exit(0);
     }
 
-    public File getConfigDir() {
+    /**
+     * returns an InputStream for a named configuration. That stream
+     * must be closed on finish.
+     */
+    public ConfigurationContainer createConfigurationContainer(String configName) {
+        return new ConfigurationContainer(new File(getConfigDir(), configName));        
+    }
+        
+    public String getConfigurationDirectoryInfo() {
+        return getConfigDir().getAbsolutePath();
+    }
+    
+    private File getConfigDir() {
 	if (_configDir != null) {
 	    return _configDir;
 	}
@@ -656,17 +742,13 @@ public class HenPlus implements Interruptable {
 	_configDir = _configDir.getAbsoluteFile();
 	try {
 	    _configDir = _configDir.getCanonicalFile();
-	} catch (IOException ign) {}
+	} catch (IOException ign) { /* ign */ }
 	
 	if (!_quiet) {
 	    System.err.println("henplus config at " + _configDir);
 	}
 	return _configDir;
-    }
-
-    private String getHistoryLocation() {
-	return getConfigDir().getAbsolutePath() + File.separator + "history";
-    }
+    }    
 }
 
 /*
