@@ -1,7 +1,7 @@
 /*
  * This is free software, licensed under the Gnu Public License (GPL)
  * get a copy from <http://www.gnu.org/licenses/gpl.html>
- * @version $Id: SQLMetaData.java,v 1.2 2004/01/27 18:16:33 hzeller Exp $ 
+ * @version $Id: SQLMetaData.java,v 1.4 2004-06-07 08:31:56 hzeller Exp $ 
  * @author <a href="mailto:martin.grotzke@javakaffee.de">Martin Grotzke</a>
  */
 package henplus;
@@ -12,24 +12,19 @@ import java.util.SortedSet;
 import java.util.TreeSet;
 
 public final class SQLMetaData {
-    
     public static final int NOT_INITIALIZED = -1;
     
-    private SortedSet /*<Table>*/ _tables;
+    private SortedSet/*<Table>*/ _tables;
     
     public SQLMetaData() {
         _tables = new TreeSet();
     }
 
-    /**
-     * @return
-     */
-    public SortedSet getTables() {
+    public SortedSet/*<Table>*/ getTables() {
         return _tables;
     }
     
     public void addTable(Table table) {
         _tables.add(table);
     }
-
 }
