@@ -15,12 +15,10 @@ import henplus.SigIntHandler;
 import henplus.property.PropertyHolder;
 import henplus.property.BooleanPropertyHolder;
 import henplus.view.util.NameCompleter;
+import henplus.view.util.CancelWriter;
 
-import java.sql.CallableStatement;
 import java.sql.ResultSet;
-import java.sql.SQLException;
 import java.sql.Statement;
-import java.sql.Types;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -33,9 +31,9 @@ import java.util.StringTokenizer;
  * document me.
  */
 public class SQLCommand extends AbstractCommand {
-    private static final boolean verbose = false; // debug.
+    private static final boolean verbose = HenPlus.verbose;
     private final static String[] TABLE_COMPLETER_KEYWORD = {
-	"FROM", "INTO", "UPDATE", "TABLE", /*create index*/"ON"
+	"FROM", "INTO", "UPDATE", "TABLE", "ALIAS", "VIEW", /*create index*/"ON"
     };
     
     /**
@@ -48,7 +46,7 @@ public class SQLCommand extends AbstractCommand {
 	    "insert", "update", "delete",
 	    "create", "alter",  "drop",
 	    "commit", "rollback",
-            "call-procedure",
+            /*"call-procedure", test */
 	    // we support _any_ string, that is not part of the
 	    // henplus buildin-stuff; the following empty string flags this.
 	    ""
@@ -56,11 +54,20 @@ public class SQLCommand extends AbstractCommand {
     }
 
     private final ListUserObjectsCommand tableCompleter;
+    private Statement _stmt;
     private String _columnDelimiter;
     private int    _rowLimit;
     private boolean _showHeader;
     private boolean _showFooter;
-
+    private volatile boolean _running;
+    private StatementCanceller _statementCanceller;
+    protected SQLCommand(ListUserObjectsCommand tc) {
+        _columnDelimiter = "|";
+        _rowLimit = 2000;
+        tableCompleter = tc;
+    }
+    private LongRunningTimeDisplay _longRunningDisplay;
+    
     public SQLCommand(ListUserObjectsCommand tc, PropertyRegistry registry) {
 	tableCompleter = tc;
         _columnDelimiter = "|";
@@ -75,6 +82,10 @@ public class SQLCommand extends AbstractCommand {
                                   new ShowHeaderProperty());
         registry.registerProperty("sql-result-showfooter",
                                   new ShowFooterProperty());
+        _statementCanceller = new StatementCanceller(new CurrentStatementCancelTarget());
+        new Thread(_statementCanceller).start();
+        _longRunningDisplay = new LongRunningTimeDisplay("statement running", 30000);
+        new Thread(_longRunningDisplay).start();
     }
 
     /**
@@ -97,15 +108,18 @@ public class SQLCommand extends AbstractCommand {
 	if (command.startsWith("COMMIT")
 	    || command.startsWith("ROLLBACK"))
 	    return true;
-	// FIXME: this is a very dumb parser. 
+	// FIXME: this is a very dumb 'parser'. 
         // i.e. string literals are not considered.
-	boolean anyProcedure = ((command.startsWith("CREATE")
-				 || command.startsWith("REPLACE"))
-				&&
-				((containsWord(command, "PROCEDURE")
-				 || (containsWord(command, "FUNCTION"))
-				 || (containsWord(command, "PACKAGE"))
-                                  || (containsWord(command, "TRIGGER")))));
+	boolean anyProcedure = (command.startsWith("BEGIN")
+                                || command.startsWith("DECLARE")
+                                || ((command.startsWith("CREATE")
+                                     || command.startsWith("REPLACE"))
+                                    &&
+                                    ((containsWord(command, "PROCEDURE")
+                                      || (containsWord(command, "FUNCTION"))
+                                      || (containsWord(command, "PACKAGE"))
+                                      || (containsWord(command, "TRIGGER"))))));
+
 	if (!anyProcedure && command.endsWith(";")) return true;
 	// sqlplus is complete on a single '/' on a line.
 	if (command.length() >= 3) {
@@ -148,6 +162,29 @@ public class SQLCommand extends AbstractCommand {
         return _showFooter;
     }
 
+    /** 
+     * A statement cancel target that accesses the instance
+     * wide statement.
+     */
+    private final class CurrentStatementCancelTarget
+        implements StatementCanceller.CancelTarget {
+        public void cancelRunningStatement() {
+            try {
+                HenPlus.msg().println("cancel statement...");
+                HenPlus.msg().flush();
+                CancelWriter info = new CancelWriter(HenPlus.msg());
+                info.print("please wait");
+                _stmt.cancel();
+                info.cancel();
+                HenPlus.msg().println("done.");
+                _running = false;
+            }
+            catch (Exception e) {
+                if (verbose) e.printStackTrace();
+            }
+        }
+    }
+
     /**
      * looks, if this word is contained in 'all', preceeded and followed by
      * a whitespace.
@@ -164,27 +201,26 @@ public class SQLCommand extends AbstractCommand {
      * execute the command given.
      */
     public int execute(SQLSession session, String cmd, String param) {
-	Statement stmt = null;
-	ResultSet rset = null;
 	String command = cmd + " " + param;
-        boolean background = false;
+        //boolean background = false;
 
 	if (command.endsWith("/")) {
 	    command = command.substring(0, command.length()-1);
 	}
 
-        if (command.endsWith("&")) {
-            command = command.substring(0, command.length()-1);
-            HenPlus.msg().println("## executing command in the background not yet supported");
-            background = true;
-        }
+//        if (command.endsWith("&")) {
+//            command = command.substring(0, command.length()-1);
+//            HenPlus.msg().println("## executing command in the background not yet supported");
+//            background = true;
+//        }
 
-	long startTime = System.currentTimeMillis();
+	final long startTime = System.currentTimeMillis();
 	long lapTime  = -1;
 	long execTime = -1;
+        ResultSet rset = null;
+        _running = true;
+        SigIntHandler.getInstance().pushInterruptable(_statementCanceller);
 	try {
-// 	    SigIntHandler.getInstance()
-// 		.registerInterrupt(Thread.currentThread());
 	    if (command.startsWith("commit")) {
 		session.print("commit..");
 		session.getConnection().commit();
@@ -196,49 +232,28 @@ public class SQLCommand extends AbstractCommand {
 		session.println(".done.");
 	    }
 	    else {
-		boolean hasResultSet = false;
-		boolean hasSingleResult = false;
+                _stmt = session.createStatement();
+                try {
+                    _stmt.setFetchSize(200);
+                }
+                catch (Exception e) {
+                    /* ignore */
+                }
 
-		/* this is basically a hack around SAP-DB's tendency to
-		 * throw NullPointerExceptions, if the session timed out.
-		 */
-		for (int retry=2; retry > 0; --retry) {
-		    try {
-                        if ("call-procedure".equals(cmd)) {
-                            CallableStatement pstmt;
-                            command = "{? = call " + param + "}";
-                            pstmt = session.getConnection()
-                                .prepareCall(command);
-                            //pstmt.registerOutParameter(1, Types.REF);
-                            pstmt.registerOutParameter(1, Types.VARCHAR);
-                            pstmt.execute();
-                            //HenPlus.msg().println("call this .." + command);
-                            //pstmt.setR
-                            hasResultSet = false;
-                            hasSingleResult = true;
-                            stmt = pstmt;
-                        }
-                        else {
-                            stmt = session.createStatement();
-                            hasResultSet = stmt.execute(command);
-                        }
-			break;
-		    }
-		    catch (SQLException e) { throw e; }
-		    catch (Throwable e) {
-			if (retry == 1) {
-			    return EXEC_FAILED;
-			}
-			HenPlus.msg().println("Problem: " + e.getMessage()
-					   + "; trying reconnect...");
-			session.connect();
-		    }
-		}
-		
+                _statementCanceller.arm();
+                _longRunningDisplay.arm();
+                final boolean hasResultSet = _stmt.execute(command);
+                _longRunningDisplay.disarm();
+                
+                if (!_running) {
+                    HenPlus.msg().println("cancelled");
+                    return SUCCESS;
+                }
+                
 		if (hasResultSet) {
-		    rset = stmt.getResultSet();
+                    rset = _stmt.getResultSet();
 		    ResultSetRenderer renderer;
-		    renderer = new ResultSetRenderer(rset, 
+		    renderer = new ResultSetRenderer(rset,
                                                      getColumnDelimiter(),
                                                      isShowHeader(),
                                                      isShowFooter(),
@@ -246,21 +261,19 @@ public class SQLCommand extends AbstractCommand {
                                                      HenPlus.out());
 		    SigIntHandler.getInstance().pushInterruptable(renderer);
 		    int rows = renderer.execute();
+		    SigIntHandler.getInstance().popInterruptable();
 		    if (renderer.limitReached()) {
 			session.println("limit of " + getRowLimit() + 
                                         " rows reached ..");
 			session.print("> ");
 		    }
 		    session.print(rows + " row" + 
-				     ((rows == 1)? "" : "s")
-				     + " in result");
+                                  ((rows == 1)? "" : "s")
+                                  + " in result");
 		    lapTime = renderer.getFirstRowTime() - startTime;
 		}
-                else if (hasSingleResult) {
-                    HenPlus.msg().println(((CallableStatement)stmt).getString(1));
-                }
 		else {
-		    int updateCount = stmt.getUpdateCount();
+		    int updateCount = _stmt.getUpdateCount();
 		    if (updateCount >= 0) {
 			session.print("affected "+updateCount+" rows");
 		    }
@@ -309,8 +322,11 @@ public class SQLCommand extends AbstractCommand {
 	    return EXEC_FAILED;
 	}
 	finally {
-	    try { if (rset != null) rset.close(); } catch (Exception e) {}
-	    try { if (stmt != null) stmt.close(); } catch (Exception e) {}
+            _statementCanceller.disarm();
+            _longRunningDisplay.disarm();
+            try { if (rset != null) rset.close(); } catch (Exception e) {}
+            try { if (_stmt != null) _stmt.close(); } catch (Exception e) {}
+            SigIntHandler.getInstance().popInterruptable();
 	}
     }
 
@@ -348,12 +364,19 @@ public class SQLCommand extends AbstractCommand {
 	else if (canonCmd.indexOf("INSERT") >= 0) {
 	    endTabMatch = canonCmd.indexOf ("(");
 	}
-	else {
-	    endTabMatch = canonCmd.indexOf ("WHERE");
-	    if (endTabMatch < 0) {
-		endTabMatch = canonCmd.indexOf (";");
-	    }
+	else if (canonCmd.indexOf ("WHERE") >= 0) {
+	    endTabMatch = canonCmd.indexOf("WHERE");
 	}
+	else if (canonCmd.indexOf ("ORDER BY") >= 0) {
+	    endTabMatch = canonCmd.indexOf("ORDER BY");
+	}
+	else if (canonCmd.indexOf ("GROUP BY") >= 0) {
+	    endTabMatch = canonCmd.indexOf("GROUP BY");
+	}
+        if (endTabMatch < 0) {
+            endTabMatch = canonCmd.indexOf (";");
+        }
+
 	if (endTabMatch > tableMatch) {
             /*
              * column completion for the tables mentioned between in the
@@ -472,7 +495,11 @@ public class SQLCommand extends AbstractCommand {
 	}
 	return result;
     }
-    
+
+    public void shutdown() {
+        _statementCanceller.stopThread();
+    }
+
     public String getSynopsis(String cmd) { 
 	cmd = cmd.toLowerCase();
 	String syn = null;
@@ -489,7 +516,7 @@ public class SQLCommand extends AbstractCommand {
 	    syn="update <table> set <column>=<value>[,...] [ where <where-clause> ]";
 	}
 	else if ("drop".equals(cmd)) {
-	    syn="drop <table|index|view|...>";
+	    syn="drop <table|index|view|alias|...>";
 	}
 	else if ("commit".equals(cmd))   { syn = cmd; }
 	else if ("rollback".equals(cmd)) { syn = cmd; }
@@ -501,7 +528,7 @@ public class SQLCommand extends AbstractCommand {
 	dsc="\t'" + cmd + "': this is not a build-in command, so would be\n"
 	    + "\tconsidered as SQL-command and handed over to the JDBC-driver.\n"
 	    + "\tHowever, I don't know anything about its syntax. RTFSQLM.\n"
-	    + "\ttry <http://www.google.de/search?q=sql+syntax+" + cmd + ">";
+	    + "\ttry <http://www.google.com/search?q=sql+syntax+" + cmd + ">";
 	cmd = cmd.toLowerCase();
 	if ("select".equals(cmd)) {
 	    dsc="\tselect from tables.";
