@@ -1,7 +1,7 @@
 /*
  * This is free software, licensed under the Gnu Public License (GPL)
  * get a copy from <http://www.gnu.org/licenses/gpl.html>
- * $Id: BooleanPropertyHolder.java,v 1.2 2003/05/01 19:53:10 hzeller Exp $ 
+ * $Id: BooleanPropertyHolder.java,v 1.3 2004-02-03 09:53:07 hzeller Exp $ 
  * author: Henner Zeller <H.Zeller@acm.org>
  */
 package henplus.property;
@@ -22,7 +22,7 @@ public abstract class BooleanPropertyHolder extends EnumeratedPropertyHolder {
         _propertyValue = initialValue ? "true" : "false";
     }
 
-    protected void enumeratedPropertyChanged(int index, String value) 
+    protected final void enumeratedPropertyChanged(int index, String value) 
         throws Exception 
     {
         /*
@@ -32,9 +32,10 @@ public abstract class BooleanPropertyHolder extends EnumeratedPropertyHolder {
     }
 
     /**
-     * to be overridden to get informed of the boolean change
+     * to be overridden to get informed of the boolean change. Throw
+     * an Exception if this change is not possible.
      */
-    public abstract void booleanPropertyChanged(boolean val) 
+    public abstract void booleanPropertyChanged(boolean newValue) 
         throws Exception;
 }
 /*
